function [SCORE]= IMS2017_meas_score_May2017(RX,gBB,PARAM)

% function [SCORE]= IMS2017_meas_score(RX,gBB,PARAM)
% This function takes as inputs:
%   RX - Structure array containing, among other fields, the measured output of the PA.
%   gBB - The original generated dual-band signal (reference signal).
%   PARAM - Structure array containing information on the most relevant
%   remoteUPCLAb parameters.
%
% The function replies with a structure array containing the SCORE for a given signal taking into
% account:
%       - The average output power (dBm).
%       - The power efficiency of the PA (%). 
%       - The ACPR for both OFDM signals (dB).
%       - The NMSE for both OFDM signals (dB).
%
% Created by Pere L. Gilabert & Gabriel Montoro  
% Date: February 2017
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Filtering and Alignment 
yBB_raw=RX.yBB;
yBB=dpd_FFTFilter(PARAM.CLK.CLKBB,0,PARAM.DEMOD.BaseBandBW,yBB_raw,'LP');

[u_ofdm1,u_ofdm2]=demod_dual_BB(gBB,PARAM);
[y_ofdm1,y_ofdm2]=demod_dual_BB(yBB,PARAM);

[y_ofdm1,u_ofdm1] = align_circular_signal(y_ofdm1,u_ofdm1);
[y_ofdm2,u_ofdm2] = align_circular_signal(y_ofdm2,u_ofdm2);

%% NMSE 
NMSE.nmse_BB=dpd_Qmeasurements(gBB,yBB,'NMSE');
NMSE.nmse_Band1=dpd_Qmeasurements(u_ofdm1,y_ofdm1,'NMSE'); 
NMSE.nmse_Band2=dpd_Qmeasurements(u_ofdm2,y_ofdm2,'NMSE');
SCORE.NMSE.nmse_Band1=NMSE.nmse_Band1; SCORE.NMSE.nmse_Band2=NMSE.nmse_Band2; SCORE.NMSE.nmse_BB=NMSE.nmse_BB;

%% ACPR 
[ACPR_Band1,POWER_center1]=dpd_acpr(PARAM,y_ofdm1,'BB',PARAM.MOD.bw_lte1*1.05);
[ACPR_Band2,POWER_center2]=dpd_acpr(PARAM,y_ofdm2,'BB',PARAM.MOD.bw_lte2*1.05); 
ACPR.ACPR_Band1=ACPR_Band1; ACPR.ACPR_Band2=ACPR_Band2;
SCORE.ACPR.ACPR_Band1=ACPR.ACPR_Band1; SCORE.ACPR.ACPR_Band2=ACPR.ACPR_Band2;
Diff_pow=abs(POWER_center1-POWER_center2);

%% SCORE
min_ACPR=-45;
min_NMSE=-33;
min_Pout=20;

score_value=mean([min_ACPR-max(ACPR_Band1(2:3)), min_ACPR-max(ACPR_Band2(2:3))])+...
    mean([min_NMSE-NMSE.nmse_Band1, min_NMSE-NMSE.nmse_Band2])*0.5*min([min_ACPR-max(ACPR_Band1(2:3)), min_ACPR-max(ACPR_Band2(2:3))]>=0)+...
    (RX.RFpower.meanpower_dBm-min_Pout)*10*min([min_ACPR-max(ACPR_Band1(2:3)), min_ACPR-max(ACPR_Band2(2:3))]>=0)*min([min_NMSE-NMSE.nmse_Band1, min_NMSE-NMSE.nmse_Band2]>=0)*((Diff_pow<0.5)&&(Diff_pow>=0))+...
    RX.RFpower.ETAPA_perc*min([min_ACPR-max(ACPR_Band1(2:3)), min_ACPR-max(ACPR_Band2(2:3))]>=0)*min([min_NMSE-NMSE.nmse_Band1, min_NMSE-NMSE.nmse_Band2]>=0)*((Diff_pow<0.5)&&(Diff_pow>=0));

SCORE.score_value=score_value;

%% PRINT RESULTS
fprintf('\n ************* RESULTS ******************');
fprintf('\n     Mean Output Power (dBm) =  %2.2f',RX.RFpower.meanpower_dBm);
fprintf('\n     Mean PA Power Efficiency (perc.) =  %2.1f',RX.RFpower.ETAPA_perc);
fprintf('\n     ACPR_Band1 (dB) =  %2.1f',max(ACPR_Band1(2:3)));
fprintf('\n     ACPR_Band2 (dB) =  %2.1f',max(ACPR_Band2(2:3)));
fprintf('\n     NMSE_OFDM_Band 1 (dB) =  %2.1f',NMSE.nmse_Band1);
fprintf('\n     NMSE_OFDM_Band 2 (dB) =  %2.1f',NMSE.nmse_Band2);
fprintf('\n     TOTAL SCORE=  %2.1f',score_value);
fprintf('\n*****************************************\n     ');

end

function [y]=dpd_FFTFilter(Fs,Fo,BWa,x,type)

L=length(x);

switch type
case 'LP'
      
    BW=ceil(L*BWa/Fs);
      
    plantilla_fft=zeros(L,1);    
    plantilla_fft(1:1:BW,1)=ones(BW,1);
    plantilla_fft(L-BW+1:1:L,1)=ones(BW,1);
    
    tf=fft(x); 
    y=ifft(plantilla_fft.*tf);
    
    %disp(sprintf('input_filterLP= %f   output_filterLP= %f',max(abs(x)),max(abs(y)) ));
    
case 'BP' 
    BW=ceil(L*BWa/Fs);
    Wo=round(L*Fo/Fs);
    W1=round(Wo-0.5*BW);
        
    plantilla_fft=zeros(L,1);    
    plantilla_fft(1:1:(W1+BW))=[zeros(W1,1);ones(BW,1)];
    plantilla_fft(end:-1:end+2-(W1+BW),1)=[zeros(W1-1,1);ones(BW,1)];
        
    tf=fft(x);
    y=ifft(plantilla_fft.*tf);
    
    
case 'COMPLEX_BP' 
    BW=floor(L*BWa/Fs)+1;
        
        if (L*Fo/Fs>0) 
         Wo=ceil(L*Fo/Fs);
        else
         Wo=floor(L*Fo/Fs)+L; 
        end
        
    W1=round(Wo-0.5*BW); 
    
    plantilla_fft=zeros(L,1);
    plantilla_fft(W1+1:1:W1+BW,1)=ones(BW,1);
    
    tf=fft(x);
    y=ifft(plantilla_fft.*tf);
otherwise
end


if (isreal(x)==1)
y=real(y);    
end

end

function [y_ofdm1,y_ofdm2]=demod_dual_BB(yBB,PARAM)

yBB1=dpd_FFTFilter(PARAM.CLK.CLKBB,PARAM.MOD.f_ofdm1,PARAM.MOD.bw_lte1*5,yBB,'COMPLEX_BP');
yBB2=dpd_FFTFilter(PARAM.CLK.CLKBB,PARAM.MOD.f_ofdm2,PARAM.MOD.bw_lte2*5,yBB,'COMPLEX_BP');
t=[0:1:PARAM.L.LBB-1]';
omega1=2*pi*PARAM.MOD.f_ofdm1/PARAM.CLK.CLKBB;
omega2=2*pi*PARAM.MOD.f_ofdm2/PARAM.CLK.CLKBB;
y_ofdm1 = yBB1.*exp(-1j*omega1*t);
y_ofdm2 = yBB2.*exp(-1j*omega2*t);
end

function [Qmeasurement]=dpd_Qmeasurements(data1,data2,type)

error=data1-data2;

ndata1=dpd_normalization(data1,'RMS');
ndata2=dpd_normalization(data2,'RMS');
nerror=ndata1-ndata2;

switch type
    case 'NONORM+NMSE'
    num=mean( abs(error).^2);
    den=mean( abs(data1).^2 );
    Qmeasurement=10*log10(num/den);
    
    case {'NMSE','NORM+NMSE'}
    num=mean( abs(nerror).^2);
    den=mean( abs(ndata1).^2 );
    Qmeasurement=10*log10(num/den);

    case 'NMSE_LIN'
    num=mean( abs(nerror).^2);
    den=mean( abs(ndata1).^2 );
    Qmeasurement=num/den;
       
    case 'SNR'
    num=mean( abs(ndata1).^2);
    den=mean( abs(nerror).^2 );
    Qmeasurement=10*log10(num/den);

    case 'EVM'
    num=sqrt(mean(abs(nerror).^2));
    den=max(abs(ndata1));
    Qmeasurement=100*num/den;
    
    case 'NONORM+INMSE'
    num=abs(error).^2;
    den=mean(abs(data1).^2);
    Qmeasurement=(num./den);
    Qmeasurement=10*log10(Qmeasurement);
    
    
    case 'NORM+INMSE'
    num=abs(nerror).^2;
    den=mean(abs(ndata1).^2);
    Qmeasurement=(num./den);
    Qmeasurement=10*log10(Qmeasurement);
    
        
    case 'DNMSE' % dynamic NMSE
    range=5;
    N=floor(length(ndata1)/range);
    for n=0:(N-1)
        n1=1+n*range;
        n2=n1+range-1;
        
        num=mean( abs(nerror(n1:1:n2)).^2);
        den=mean( abs(ndata1(n1:1:n2)).^2 );
        nmse=10*log10(num/den);
        Qmeasurement(n+1,1)=nmse;
    end

    
    case {'NONORM+PNMSE'} % Percentatge NMSE
        num=abs(nerror).^2;
        den=abs(ndata1).^2;
        buffer=mean(num./den);
        Qmeasurement=10*log10(buffer);
    
    otherwise
end

end

function [ACPR,POWER_center]=dpd_acpr(PARAM,x,type,BWa)

switch type
           
case 'RF'
    Fs=PARAM.DEMOD.SamplingRate;
    Fo=[PARAM.DEMOD.CenterFreq-2*BWa,PARAM.DEMOD.CenterFreq-BWa,PARAM.DEMOD.CenterFreq+BWa,PARAM.DEMOD.CenterFreq+2*BWa]; 
    
    POWER_center=10*log10(mean(abs( dpd_FFTFilter(Fs,PARAM.DEMOD.CenterFreq,BWa,x,'BP') ).^2/50))+30;
    for n=1:1:4
        buffer=dpd_FFTFilter(Fs,Fo(n),BWa,x,'BP');
        ACPR(n)=10*log10(mean(abs(buffer).^2/50))+30-POWER_center;
    end
   
case 'BB'
    Fs=PARAM.CLK.CLKBB;
    Fo=[-2*BWa,-BWa,BWa,2*BWa];
    
    POWER_center=10*log10(mean(abs( dpd_FFTFilter(Fs,0,0.5*BWa,x,'LP') ).^2/50))+30;
    for n=1:1:4
        buffer=dpd_FFTFilter(Fs,Fo(n),BWa,x,'COMPLEX_BP');
        ACPR(n)=10*log10(mean(abs(buffer).^2/50))+30-POWER_center;
    end

otherwise
end;
       
end

function [y]=dpd_normalization(x,type)


switch type

case 'NONE' 
y=x;    
        
case 'RMS'
rms=sqrt( mean( abs(x).^2 ) );
y=x/rms;
   
case 'PEAK' 
peak=max(abs(x));
y=x/peak;

otherwise
end
end

function [x_a,y_a,del_t] = align_circular_signal(x,y)
% [x_a,y_a] = align_circular_signal(x,y)
%
% Aligns the signals x and y considering that they are circular (periodic).
% The procedure is iterative, default 10 iterations.

%
% Author: Filipe Barradas
% Date: November 2013

%% Coarse allignment
Hf = fft(x).*conj(fft(y));
H = ifft(Hf);
[~,ind_m] = max(abs(H));

x_a = circshift(x,-(ind_m-1));
h = (x_a'*y)./(x_a'*x_a);
h = exp(1j*angle(h));
x_a = h.*x_a;

%% Fine Alignment
N = length(x_a);
f = fftshift((-1/2+1/N:1/N:1/2).');

tol = 1e-5*norm(x)*norm(y);

for k1 = 1:10
    d = delay_estimator(x_a,y,tol);
    del = exp(-1j*2*pi*f*d);
    x_a = ifft(fft(x_a).*conj(del));
    del_t = ind_m + d;
end
h = (x_a'*y)./(x_a'*x_a);
h = exp(1j*angle(h));
x_a = h.*x_a;
y_a = y;
end

function d = delay_estimator(x,y,tol)
%

%
% Author: Filipe Barradas
% Date: November 2013

%% Estimate delay
N = length(x);

R3 = ifft(fft(x).*conj(fft(y)))/N;

R = -real(R3);
lags = [(0:round(N/2)-1)';(-round(N/2):-1)'];

R_m = min(R);

pick = (abs(R-R_m)<=tol);

ind = (1:N)';
ind = ind(pick);
ind = ind(1);

d = lags(ind) - 1/2*(R(mod(ind-1+1,N)+1) - R(mod(ind-1-1,N)+1))/(R(mod(ind-1+1,N)+1) - 2*R(ind) + R(mod(ind-1-1,N)+1));
if isnan(d) || isinf(d)
    d = lags(ind);
end
end
